/*
 * fuc: message.js v0.1.0
 */

import $ from 'jquery';

const prefixCls = 'fuc-message';
const now = Date.now();
let seed = 0;
let defaultDuration = 3;
let defaultContainer = 'body';
let defaultTop;

function noop() {
}

function getUuid() {
  return `message_${now}_${seed++}`;
}

function notice(content, duration = defaultDuration, type, onClose = noop) {
  let $message = $('[data-toggle="message"]');
  const uuid = getUuid();
  const iconType = ({
    info: 'info-circle',
    success: 'check-circle',
    error: 'close-circle-o',
    warning: 'exclamation-circle',
    loading: 'loading',
  })[type];
  const style = typeof defaultTop === 'number' ? `top:${defaultTop}px` : '';

  if (typeof duration === 'function') {
    onClose = duration;
    duration = defaultDuration;
  }

  if ($message.length === 0) {
        $message = $(`<div class="${prefixCls}" data-toggle="message" style="${style}"></div>`);
        $(defaultContainer).append($message);
    }

  $message.append(`<div class="${prefixCls}-notice move-up-enter" id="${uuid}">
          <div class="${prefixCls}-notice-content">
            <div class="${prefixCls}-custom-content ${prefixCls}-${type}">
              <i class="iconfont icon-${iconType}"></i>
              <span>${content}</span>
            </div>
          </div>
        </div>`)

  function destory() {
    const $item = $(`#${uuid}`)
    $item.addClass('move-up-leave')
    setTimeout(() => {
      $item.remove()
      onClose()
    }, 300)
  }

  if (duration === 0) {
    return () => {
      destory()
    }
  }

  setTimeout(() => {
    destory()
  }, duration * 1000)
}

const Message = $.Message = {
  info(content, duration, onClose) {
    return notice(content, duration, 'info', onClose);
  },
  success(content, duration, onClose) {
    return notice(content, duration, 'success', onClose);
  },
  error(content, duration, onClose) {
    return notice(content, duration, 'error', onClose);
  },
  warning(content, duration, onClose) {
    return notice(content, duration, 'warning', onClose);
  },
  loading(content, duration, onClose) {
    return notice(content, duration, 'loading', onClose);
  },
  config(options) {
    if (options.top !== undefined) {
      defaultTop = options.top;
    }
    if (options.duration !== undefined) {
      defaultDuration = options.duration;
    }
    if (options.getContainer !== undefined) {
      defaultContainer = options.getContainer;
    }
  },
};

export default Message;
